// ================================================================================
// (c) 2012 Altera Corporation. All rights reserved.
// Altera products are protected under numerous U.S. and foreign patents, maskwork
// rights, copyrights and other intellectual property laws.
// 
// This reference design file, and your use thereof, is subject to and governed
// by the terms and conditions of the applicable Altera Reference Design License
// Agreement (either as signed by you, agreed by you upon download or as a
// "click-through" agreement upon installation andor found at www.altera.com).
// By using this reference design file, you indicate your acceptance of such terms
// and conditions between you and Altera Corporation.  In the event that you do
// not agree with such terms and conditions, you may not use the reference design
// file and please promptly destroy any copies you have made.
// 
// This reference design file is being provided on an "as-is" basis and as an
// accommodation and therefore all warranties, representations or guarantees of
// any kind (whether express, implied or statutory) including, without limitation,
// warranties of merchantability, non-infringement, or fitness for a particular
// purpose, are specifically disclaimed.  By making this reference design file
// available, Altera expressly does not recommend, suggest or require that this
// reference design file be used in combination with any other product not
// provided by Altera.
// 31 Dec 2012 - SAM
//             - Verilog 2001 I/O to match other blocks
//             - disclaimer added
//             - metastability registers for crc_error w/export
module cb_interface(
		    // Inputs
		    input 	 sreset, 
		    input 	 clk, 
		    input 	 pr_start,
		    input [15:0] pr_data,
		    // Outputs
		    output 	 pr_ready, 
		    output 	 pr_error, 
		    output 	 crc_error, 
		    output 	 pr_done, 
		    output 	 really_done, 
		    output 	 ready_for_pr
		    );

   reg [2:0] 			 crc_error_reg = 'b0;
   (* keep *) wire [15:0] pr_data_w = pr_data;
   (* keep *)   wire 			 pr_request_w, 
					 crc_error_w,
					 pr_error_w,
					 pr_ready_w,
					 pr_done_w,
					 pr_start_w;

   assign pr_error = pr_error_w;
   assign pr_ready = pr_ready_w;
   assign pr_done = pr_done_w;
   assign pr_start_w = pr_start;
      
   assign crc_error = crc_error_reg[2];

pr_host m_pr_host 
(
	.sreset       ( sreset        ),
	.clk          ( clk           ),
	.pr_start     ( pr_start_w    ),
	.pr_ready     ( pr_ready_w    ),
	.really_done  ( really_done   ),
	.ready_for_pr ( ready_for_pr  ), 
	.pr_request   ( pr_request_w  ),
	.pr_error     ( pr_error_w    ),
	.crc_error    ( crc_error     ),
	.pr_done      ( pr_done_w     )
);
	
   // Stratix V PRBLOCK
   stratixv_prblock m_prblock
     (
	.clk       ( clk          ),
	.corectl   ( 1'b1         ),
	.prrequest ( pr_request_w ),
	.data      ( pr_data_w    ),
	.error     ( pr_error_w   ),
	.ready     ( pr_ready_w   ),
	.done      ( pr_done_w    )
);	

   // Stratix V CRCBLOCK
   stratixv_crcblock m_crcblock
     (
      .clk      ( clk         ),
      .shiftnld ( 1'b1        ),		
      .crcerror ( crc_error_w )
      );
   // latch the crc_error since it is really asynchronous

always @(posedge clk)
begin
	if (sreset)
	begin
		crc_error_reg <= 'b0;
	end
	else
	begin
		crc_error_reg <= {crc_error_reg[1:0],crc_error_w};
	end
end



endmodule

