// ================================================================================
// (c) 2012 Altera Corporation. All rights reserved.
// Altera products are protected under numerous U.S. and foreign patents, maskwork
// rights, copyrights and other intellectual property laws.
// 
// This reference design file, and your use thereof, is subject to and governed
// by the terms and conditions of the applicable Altera Reference Design License
// Agreement (either as signed by you, agreed by you upon download or as a
// "click-through" agreement upon installation andor found at www.altera.com).
// By using this reference design file, you indicate your acceptance of such terms
// and conditions between you and Altera Corporation.  In the event that you do
// not agree with such terms and conditions, you may not use the reference design
// file and please promptly destroy any copies you have made.
// 
// This reference design file is being provided on an "as-is" basis and as an
// accommodation and therefore all warranties, representations or guarantees of
// any kind (whether express, implied or statutory) including, without limitation,
// warranties of merchantability, non-infringement, or fitness for a particular
// purpose, are specifically disclaimed.  By making this reference design file
// available, Altera expressly does not recommend, suggest or require that this
// reference design file be used in combination with any other product not
// provided by Altera.
//
// Revision History
// 31 Dec 2012 - SAM
//             - all states return to idle so reset doesn't have to be issued
//             - added disclaimer
// 3 Jan 2013  - SAM
//             - added allow_pr_start to initiate state machine
//             - cleanup of state block signals and minor edits in state machine
module main_flow(
		 // Inputs
		 input 	allow_pr_start, 
		 input 	clk, 
		 input 	crc_error, 
		 input 	pr_really_done, 
		 input 	pr_error, 
		 input 	ready_for_pr,
		 input 	sreset,
		 // Outputs
		 output pr_freeze, 
		 output pr_start, 
		 output testing_done, 
		 output error_flag_pr
		 );

   reg [1:0] 		test_state;
   reg 			pr_freeze_reg, 
			pr_start_reg, 
			testing_done_reg, 
			error_flag_pr_reg;

   wire 		any_error = pr_error || crc_error;
   assign pr_freeze = pr_freeze_reg;
   assign pr_start = pr_start_reg;
   assign testing_done = testing_done_reg;
   assign error_flag_pr = error_flag_pr_reg;
   
   parameter 
     IDLE        = 0,
     SWITCH_TO_B = 1,
     DONE        = 2,
     ERROR_PR    = 3;

   always @(posedge clk)
     begin
	if (sreset) begin
	   test_state        <= IDLE;
	   pr_freeze_reg     <= 1'b0;
	   pr_start_reg      <= 1'b0;
	   testing_done_reg  <= 1'b0;
	   error_flag_pr_reg <= 1'b0;
	end
	else begin
	   case (test_state)
	     IDLE:
	       begin
		  pr_freeze_reg     <= 1'b0;
		  pr_start_reg      <= 1'b0;
		  if (allow_pr_start) begin
		     testing_done_reg  <= 1'b0;
		     test_state        <= SWITCH_TO_B;
		     error_flag_pr_reg <= 1'b0;
		  end
		  else begin
		     test_state        <= IDLE;
	       	     testing_done_reg  <= testing_done_reg;
		     error_flag_pr_reg <= error_flag_pr_reg;
		  end
	       end // case: IDLE
	     
	     SWITCH_TO_B:
	       begin
		  testing_done_reg  <= 1'b0;
		  error_flag_pr_reg <= 1'b0;
		  if ((ready_for_pr) && (!pr_really_done))
		    begin
		       pr_freeze_reg <= 1'b1;
		       pr_start_reg  <= 1'b1;
		       test_state    <= SWITCH_TO_B;
		    end
		  else if (pr_really_done)
		    begin
		       pr_freeze_reg <= 1'b0;
		       pr_start_reg  <= 1'b0;
		       test_state    <= DONE;					
		    end
		  test_state <= any_error ? ERROR_PR : ((pr_really_done ) ? DONE : SWITCH_TO_B);
	       end // case: SWITCH_TO_B
	     
	     DONE:	
	       begin
		  pr_freeze_reg     <= 1'b0;
		  pr_start_reg      <= 1'b0;
		  testing_done_reg  <= 1'b1;
		  error_flag_pr_reg <= 1'b0;
		  test_state        <= IDLE;
	       end
	     
	     ERROR_PR:
	       begin	
		  pr_freeze_reg     <= 1'b0;
		  pr_start_reg      <= 1'b0;
		  testing_done_reg  <= 1'b0;
		  error_flag_pr_reg <= 1'b1;
		  test_state        <= IDLE;
	       end
	     
	     default: 
	       begin
		  pr_freeze_reg     <= 1'b0;
		  pr_start_reg      <= 1'b0;
		  testing_done_reg  <= 1'b0;
		  error_flag_pr_reg <= 1'b0;
		  test_state        <= IDLE;
	       end
	   endcase // case (test_state)
	end // else: !if(sreset)
     end // always @ (posedge clk)
   	
	endmodule // main_flow




