// ================================================================================
// (c) 2012 Altera Corporation. All rights reserved.
// Altera products are protected under numerous U.S. and foreign patents, maskwork
// rights, copyrights and other intellectual property laws.
// 
// This reference design file, and your use thereof, is subject to and governed
// by the terms and conditions of the applicable Altera Reference Design License
// Agreement (either as signed by you, agreed by you upon download or as a
// "click-through" agreement upon installation andor found at www.altera.com).
// By using this reference design file, you indicate your acceptance of such terms
// and conditions between you and Altera Corporation.  In the event that you do
// not agree with such terms and conditions, you may not use the reference design
// file and please promptly destroy any copies you have made.
// 
// This reference design file is being provided on an "as-is" basis and as an
// accommodation and therefore all warranties, representations or guarantees of
// any kind (whether express, implied or statutory) including, without limitation,
// warranties of merchantability, non-infringement, or fitness for a particular
// purpose, are specifically disclaimed.  By making this reference design file
// available, Altera expressly does not recommend, suggest or require that this
// reference design file be used in combination with any other product not
// provided by Altera.
//
// 31 Dec 2012 - SAM
//             - added channel ability
//             - added disclaimer
//             - Verilog 2001 I/O to match other blocks
// 3 Jan 2013  - SAM
//             - added allow_pr_start connection for rom_bitstream

module pr_engine(
		 // Inputs
		 input 	sreset, 
		 input 	clk, 
		 input 	pr_start, 
		 input 	channel,
		 input 	allow_pr_start,
		 // Outputs
		 output pr_error, 
		 output crc_error, 
		 output pr_really_done, 
		 output ready_for_pr
		 );

   wire [15:0] 		bit_data;
   wire 		pr_ready, 
			pr_done,
			pr_any_status;
   
   assign pr_any_status = pr_done | crc_error | pr_error;
   
   cb_interface m_cb_interface
     (
      .sreset       ( sreset         ), 
      .clk          ( clk            ), 
      .pr_start     ( pr_start       ), 
      .pr_ready     ( pr_ready       ),
      .pr_error     ( pr_error       ),
      .crc_error    ( crc_error      ),
      .pr_done      ( pr_done        ),
      .really_done  ( pr_really_done ),
      .pr_data      ( bit_data       ),
      .ready_for_pr ( ready_for_pr   )
      );
   

   rom_bitstream m_rom_bitstream 
     (	
	.sreset         ( sreset         ),
	.channel        ( channel        ),
	.ready_for_pr   ( ready_for_pr   ),
	.clk            ( clk            ),
	.bit_data       ( bit_data       ), 
	.pr_ready       ( pr_ready       ),
	.pr_any_status  ( pr_any_status  ),
	.allow_pr_start ( allow_pr_start )
	);
   
endmodule // pr_engine

