// ================================================================================
// (c) 2012 Altera Corporation. All rights reserved.
// Altera products are protected under numerous U.S. and foreign patents, maskwork
// rights, copyrights and other intellectual property laws.
// 
// This reference design file, and your use thereof, is subject to and governed
// by the terms and conditions of the applicable Altera Reference Design License
// Agreement (either as signed by you, agreed by you upon download or as a
// "click-through" agreement upon installation andor found at www.altera.com).
// By using this reference design file, you indicate your acceptance of such terms
// and conditions between you and Altera Corporation.  In the event that you do
// not agree with such terms and conditions, you may not use the reference design
// file and please promptly destroy any copies you have made.
// 
// This reference design file is being provided on an "as-is" basis and as an
// accommodation and therefore all warranties, representations or guarantees of
// any kind (whether express, implied or statutory) including, without limitation,
// warranties of merchantability, non-infringement, or fitness for a particular
// purpose, are specifically disclaimed.  By making this reference design file
// available, Altera expressly does not recommend, suggest or require that this
// reference design file be used in combination with any other product not
// provided by Altera.

// ////////////////////////////////////////////////////////////
// module pr_host: PR HOST THAT CONTROLS INPUT SIGNALS OF PRBLOCK
//                 THIS BLOCK IN CONJUNTION WITH ROM_BITSTREAM CONTROLS PRBLOCK
////////////////////////////////////////////////////////////
// 31 Dec 2012 - SAM
//             - added default values for signals in state block
//             - Verilog 2001 input style to match other blocks
//             - disclaimer added

module pr_host (
		// Inputs
		input 	   sreset, 
		input 	   pr_start, 
		input 	   clk, 
		input 	   pr_error, 
		input 	   pr_ready, 
		input 	   pr_done, 
		input 	   crc_error,
		// Outputs
		output 	   really_done, 
		output reg ready_for_pr,
		output reg pr_request 
		);
   
   reg [4:0] 		   cnt_done_to_last_clock = 'b0;
   reg [2:0] 		   pr_state = 'b0;
   
   parameter [4:0] DONE_TO_LAST_CLOCK = 20;

   parameter [2:0] 
     IDLE                         = 0,
     WAIT_FOR_PR_READY            = 1,
     SEND_PR_DATA                 = 2,
     WAIT_DONE_TO_LAST_CLOCK      = 3,
     WAIT_PR_ERROR_TO_LAST_CLOCK  = 4,
     WAIT_CRC_ERROR_TO_LAST_CLOCK = 5,
     PR_REQUEST_DONE              = 6;

   assign really_done = (pr_state == PR_REQUEST_DONE);
   
   // THIS STATE MACHINE ONLY CONTROLS ready_for_pr AND really_done SINCE BITSTREAMS ARE SENT FROM ROM AND NOT HERE
   always @(posedge clk) 
     begin
	if (sreset)
	  begin
	     pr_state               <= IDLE;
	     ready_for_pr           <= 0;
	     pr_request             <= 0;
	     cnt_done_to_last_clock <= 0;
	  end
	else 
	  begin
	     // default values
	     pr_state               <= pr_state;
	     cnt_done_to_last_clock <= cnt_done_to_last_clock;
	     pr_request             <= pr_request;
	     ready_for_pr           <= ready_for_pr;
	     
	     case (pr_state)
	       IDLE: 
		 begin
		    ready_for_pr <= 1;
		    pr_request   <= 0;
		    if (pr_start) 
		      begin
			 pr_state <= WAIT_FOR_PR_READY;
			 cnt_done_to_last_clock <= 0;
			 pr_request <= 1;
		      end
		 end // case: IDLE
	       
	       WAIT_FOR_PR_READY: 
		 begin
		    ready_for_pr <= 0;
		    if (pr_ready) 
		      begin
			 pr_state <= SEND_PR_DATA;
		      end
		 end
	       
	       SEND_PR_DATA: 
		 begin
		    if (pr_done) 
		      begin
			 pr_state <= WAIT_DONE_TO_LAST_CLOCK;
			 pr_request <= 0;
		      end
		    else if (pr_error) 
		      begin
			 pr_request <= 0;
			 pr_state <= WAIT_PR_ERROR_TO_LAST_CLOCK;
		      end
		    else if (crc_error) 
		      begin
			 pr_request <= 0;
			 pr_state <= WAIT_CRC_ERROR_TO_LAST_CLOCK;
		      end
		 end // case: SEND_PR_DATA
	       
	       WAIT_DONE_TO_LAST_CLOCK: 
		 begin
		    if (cnt_done_to_last_clock < DONE_TO_LAST_CLOCK - 1'd1)
		      begin
			 cnt_done_to_last_clock <= cnt_done_to_last_clock + 1'd1;
		      end
		    else if (!pr_done)
		      begin
			 pr_state <= PR_REQUEST_DONE;
		      end
		 end // case: WAIT_DONE_TO_LAST_CLOCK
	       
	       WAIT_PR_ERROR_TO_LAST_CLOCK: 
		 begin
		    if (!pr_error) 
		      begin
			 pr_state <= IDLE;
		      end
		 end
	       
	       WAIT_CRC_ERROR_TO_LAST_CLOCK: 
		 begin
		    if (!crc_error) 
		      begin
			 pr_state <= IDLE;
		      end
		 end
	       
	       PR_REQUEST_DONE: 
		 begin
		    pr_state <= IDLE;
		 end
	       
	       default: 
		 begin
		    pr_state               <= IDLE;
		    ready_for_pr           <= 0;
		    pr_request             <= 0;
		    cnt_done_to_last_clock <= 0;
		 end
	     endcase // case (pr_state)
	  end // else: !if(sreset)
     end // always @ (posedge clk)
    
endmodule // pr_host

