// ================================================================================
// (c) 2012 Altera Corporation. All rights reserved.
// Altera products are protected under numerous U.S. and foreign patents, maskwork
// rights, copyrights and other intellectual property laws.
// 
// This reference design file, and your use thereof, is subject to and governed
// by the terms and conditions of the applicable Altera Reference Design License
// Agreement (either as signed by you, agreed by you upon download or as a
// "click-through" agreement upon installation andor found at www.altera.com).
// By using this reference design file, you indicate your acceptance of such terms
// and conditions between you and Altera Corporation.  In the event that you do
// not agree with such terms and conditions, you may not use the reference design
// file and please promptly destroy any copies you have made.
// 
// This reference design file is being provided on an "as-is" basis and as an
// accommodation and therefore all warranties, representations or guarantees of
// any kind (whether express, implied or statutory) including, without limitation,
// warranties of merchantability, non-infringement, or fitness for a particular
// purpose, are specifically disclaimed.  By making this reference design file
// available, Altera expressly does not recommend, suggest or require that this
// reference design file be used in combination with any other product not
// provided by Altera.
//
// 31 Dec 2012 - SAM
//             - added channel ability
//             - changed state machine to allow start without reset (allow_pr_start)
//             - minor edits to state logic

// ////////////////////////////////////////////////////////////
// module rom_bitstream: SENDS BITSTREAMS FROM ROM. 
////////////////////////////////////////////////////////////
module rom_bitstream (
		      input 		sreset, 
		      input 		ready_for_pr, 
		      input 		clk, 
		      input 		pr_ready, 
		      input 		pr_any_status, 
		      input 		channel,
		      input 		allow_pr_start,
		      output reg [15:0] bit_data 
		      );


   reg [19:0] 			    ADDRESS; // ROM is 650000 16-bit words
   wire [15:0] 			    mult_q, add_q;
   
   parameter [1:0] 
     IDLE           = 0,
     WAIT_FOR_READY = 1,
     SET_BIT_DATA   = 2;
   
   reg [1:0] 			    ROM_STATE;	
   
   always @(posedge clk) begin
      bit_data <= channel ? add_q : mult_q;
   end

   // ROM INSTANTIATION
   mult_rom_pr mult_rom_pr_inst(
				.address ( ADDRESS ),
				.clock   ( clk     ),
				.q       ( mult_q  )
				);
   
   add_rom_pr add_rom_pr_inst(
			      .address   ( ADDRESS ),
			      .clock     ( clk     ),
			      .q         ( add_q   )
			      );
   
   always @(posedge clk or posedge sreset) begin
      if (sreset) 
	begin
	   ROM_STATE <= IDLE;
	   ADDRESS   <= 'b0;
	end
      else
	begin
	   ROM_STATE <= ROM_STATE;
	   ADDRESS   <= ADDRESS;
	   
	   case (ROM_STATE)
	     IDLE: begin
		if ( !sreset & ready_for_pr ) begin
		   ROM_STATE <= allow_pr_start ? WAIT_FOR_READY : IDLE;
		   ADDRESS   <= 'b0;
		end
	     end
	     
	     WAIT_FOR_READY: begin
		if (!sreset & pr_ready) 
		  begin
		     ADDRESS   <= ADDRESS + 1'd1;
		     ROM_STATE <= SET_BIT_DATA;
		  end		
	     end
	     
	     SET_BIT_DATA: begin
		if (!sreset) begin
		   if (!pr_any_status) begin
		      ADDRESS   <= ADDRESS + 1'd1;
		      ROM_STATE <= SET_BIT_DATA;
		   end
		   else begin
		      ADDRESS   <= ADDRESS;
		      ROM_STATE <= IDLE;
		   end
		end
	     end
	     
	     default: begin
		ADDRESS   <= 'b0;
		ROM_STATE <= IDLE;
	     end
	     
	   endcase // case (ROM_STATE)
	end // else: !if(sreset)
   end // always @ (posedge clk or posedge sreset)

endmodule // rom_bitstream

