// ================================================================================
// (c) 2012 Altera Corporation. All rights reserved.
// Altera products are protected under numerous U.S. and foreign patents, maskwork
// rights, copyrights and other intellectual property laws.
// 
// This reference design file, and your use thereof, is subject to and governed
// by the terms and conditions of the applicable Altera Reference Design License
// Agreement (either as signed by you, agreed by you upon download or as a
// "click-through" agreement upon installation andor found at www.altera.com).
// By using this reference design file, you indicate your acceptance of such terms
// and conditions between you and Altera Corporation.  In the event that you do
// not agree with such terms and conditions, you may not use the reference design
// file and please promptly destroy any copies you have made.
// 
// This reference design file is being provided on an "as-is" basis and as an
// accommodation and therefore all warranties, representations or guarantees of
// any kind (whether express, implied or statutory) including, without limitation,
// warranties of merchantability, non-infringement, or fitness for a particular
// purpose, are specifically disclaimed.  By making this reference design file
// available, Altera expressly does not recommend, suggest or require that this
// reference design file be used in combination with any other product not
// provided by Altera.
//
// Revision History
// 31 Dec 2012 - S. Masepohl
//             - initial revision
//             - added channel ability to design to switch between MIFs
//             - deglitched buttons
//             - seperated pr_wrapper to other file
//             - ifdef option for Stratix V internal osciallator functionality
//             - added disclaimer
// 3 Jan 2013  - SAM
//             - really done no longer used by button_set
//             - corrected default polarity on channel and DONE signal

module top(
	   input 	 clk,
	   output [55:0] test_bus_out,
	   output 	 led1_g,                       // USER_LED_G1, pin U10
	   output 	 led1_r,                       // USER_LED_R1, pin AG30
	   input 	 user_pb1_pin,                 // USER_PB1, pin B7
	   // PR stuff
	   input 	 user_pb2_pin,                 // USER_PB2, pin C7
	   input 	 user_pb0_pin,                 // USER_PB0, pin A7
	   output reg 	 switch_PR_image_DONE = 1'b1,  // LED USER_LED_G0, pin J11
	   output reg 	 switch_PR_image_ERROR = 1'b1  // LED USER_LED_R0, pin AH28

	   );

   reg 			 pr_freeze_reg1;
`ifdef USE_INT_OSC
   wire 		 div_clk;
   wire 		 intosc;
   wire 		 intclk;
`else
   reg 			 div_clk = 'b0;
`endif
   wire [31:0] 		 data_out;
   wire [9:0] 		 data_a_in,
			 data_b_in;
   wire 		 done_w,
     			 error_flag_pr_w, 
     			 pr_freeze,
     			 switch_PR_image,
     			 areset,
			 channel,
			 really_done,
			 user_pb0,
			 user_pb2;
   
`ifdef USE_INT_OSC
   stratixv_oscillator stratixv_oscillator 
     (
      .oscena ( 1'b1   ),
      .clkout ( intclk )
      );
   assign div_clk = int_clk;
`else
   always @(posedge clk)
     div_clk = ~div_clk;
`endif
   
   assign test_bus_out = { data_a_in, data_b_in, data_out[19:0] };
   assign led1_g = channel;
   assign led1_r = ~channel;


   
   my_probe_source issp_inst0(
			      .probe      ( data_out[19:0]           ),
			      .source     ( { data_a_in, data_b_in } ),
			      .source_clk ( div_clk                  )
			      );
   pr_wrapper pr_wrapper(
			 .freeze     ( pr_freeze_reg1 ),
			 .data_a_in  ( data_a_in      ),
			 .data_b_in  ( data_b_in      ),
			 .clk        ( div_clk        ),
			 .data_out   ( data_out       )
			 );

   
   
   
   // /************************ PR Logic **************************/

   // , reset_board_reg1, reset_board_reg2;
   
   checkout_top checkout_top_inst  
     (
      .areset         ( user_pb0          ),
      .clk            ( div_clk           ),
      .testing_done   ( done_w            ),
      .pr_freeze      ( pr_freeze         ),
      .error_flag_pr  ( error_flag_pr_w   ),
      .allow_pr_start ( user_pb2          ),
      .channel        ( channel           ),
      .really_done    ( really_done       )
      );

   button_set inst_pb0(
		       .clk      ( div_clk           ),
		       .pb_n_pin ( user_pb0_pin      ),
		       .pb       ( user_pb0          ),
		       .switch   ( /*NC*/            )
		       );
   
   button_set inst_pb1(
		       .clk      ( div_clk           ),
		       .pb_n_pin ( user_pb1_pin      ),
		       .pb       ( /*NC*/            ),
		       .switch   ( channel           )
		       );

   button_set inst_pb2(
		       .clk      ( div_clk               ),
		       .pb_n_pin ( user_pb2_pin          ),
		       .pb       ( user_pb2              ),
		       .switch   ( /*NC*/                )
		       );
   
   always @(posedge div_clk)
     begin
	switch_PR_image_DONE  <= ~done_w; 
	pr_freeze_reg1        <= pr_freeze;
	switch_PR_image_ERROR <= ~error_flag_pr_w;
     end

endmodule // top
